import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_workers = click.Group('workers', help='Workers management')


@cli_workers.command('list', help='List all workers')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.pass_obj
def cli_workers_list(obj: AppConfig, remote):
    remote: Remote = helpers.get_remote(obj, remote)
    workers = remote.kontr_client.workers.list()
    helpers.generic_list(workers, params=['id', 'codename', 'state', 'url'])


@cli_workers.command('read', help='Get a worker info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_workers_read(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_read(remote.kontr_client.workers, codename)


@cli_workers.command('status', help='Get a worker status')
@click.option('-r', '--remote', required=False, help='Gets a worker status')
@click.argument('codename')
@click.pass_obj
def cli_workers_status(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    worker = helpers.read_helper(remote.kontr_client.workers, codename)
    status = worker.status.read()
    helpers.entity_printer(status)


@cli_workers.command('delete', help='Delete worker')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_workers_delete(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Removing worker: {codename}")
    remote.kontr_client.workers.delete(codename)


@cli_workers.command('edit', help='Edit worker')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_workers_edit(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_edit(remote.kontr_client.workers, codename)


@cli_workers.command('create', help='Create worker')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-n', '--name', prompt=True, required=False, help='Name of the worker')
@click.option('-U', '--url', required=False, help='Url')
@click.option('-P', '--portal_secret', required=False, help='Portal secret')
@click.option('-T', '--tags', required=False, help='Tags')
def cli_workers_create(obj, remote, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_create(remote.kontr_client.workers.create, **kwargs)




import click
import json

from kontrctl.commands.submission_files import cli_submission_files
from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_submissions = click.Group('submissions', help='Submissions management')

cli_submissions.add_command(cli_submission_files)


def remove_none(**params):
    new_params = {}
    for (k, v) in params.items():
        if v is not None:
            new_params[k] = v
    return new_params


@cli_submissions.command('list', help='List all submissions')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Sets course')
@click.option('-p', '--project', required=False, help='Sets project')
@click.option('-u', '--user', required=False, help='Sets user')
@click.option('-i', '--ids', required=False, is_flag=True, help='Get just ids')
@click.option('-s', '--state', required=False, help='Get submissions by state')
@click.option('-R', '--result', required=False, help='Get submissions by result')
@click.option('-P', '--points', required=False, help='Get points by expression (example: p > 3.0)')
@click.option('-C','--cols', required=False, help='Show just provided columns, separator is ;')
@click.option('--t-format', required=False, help='Table formats (default simple), look at the tabular projects for more')
@click.pass_obj
def cli_submissions_list(obj: AppConfig, remote=None, **kwargs):
    params = remove_none(**kwargs)
    remote: Remote = helpers.get_remote(obj, remote)
    submissions = remote.kontr_client.submissions.list(params=kwargs)
    params = ['id', 'created_at', 'result', 'points', 'state', 'user.username', 
    'course.codename', 'project.codename']
    submissions = helpers.filter_submissions(submissions, **kwargs)
    helpers.generic_list(submissions, options=kwargs, params=params)


@cli_submissions.command('read', help='Get a submission info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submissions_read(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    if submission:
        helpers.entity_printer(submission)
    else:
        print("There are no submissions for you.")


@cli_submissions.command('stats', help='Get a submission stats')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submissions_read(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    if submission:
        print(json.dumps(submission.stats().json(), indent=4))
    else:
        print("There are no submissions for you.")


@cli_submissions.command('rm', help='Delete submission')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submissions_delete(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Removing submission: {sid}")
    submission = helpers.get_submission(remote, sid)
    submission.delete()


@cli_submissions.command('cancel', help='Cancel submission')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submissions_cancel(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Canceling submission: {sid}")
    submission = helpers.get_submission(remote, sid)
    submission.cancel()


@cli_submissions.command('resubmit', help='Resubmit submission')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submissions_resubmit(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Resubmitting submission: {sid}")
    submission = helpers.get_submission(remote, sid)
    submission.resubmit()


import click
from kontr_api import resources

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_courses = click.Group('courses', help='Courses management')


@cli_courses.command('list', help='List all courses')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('--ids', default=False, is_flag=True, required=False, help='Print just ids')
@click.option('--codes', default=False, is_flag=True, required=False,
              help='Print just course codes')
@click.pass_obj
def cli_courses_list(obj: AppConfig, remote, ids, codes):
    remote: Remote = helpers.get_remote(obj, remote)
    courses = remote.kontr_client.courses.list()
    if not codes and not ids:
        helpers.generic_list(courses, params=['id', 'name', 'codename'])
        return
    for course in courses:
        if ids and codes:
            print(f"{course['codename']} {course['id']}")
        elif ids:
            print(course['id'])
        elif codes:
            print(course['codename'])


@cli_courses.command('read', help='Get a course info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_courses_read(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_read(remote.kontr_client.courses, codename)


@cli_courses.command('rm', help='Delete course')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_courses_rm(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Removing course: {codename}")
    remote.kontr_client.courses.delete(codename)


@cli_courses.command('edit', help='Edit courses config')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_courses_edit(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_edit(remote.kontr_client.courses, codename)


@cli_courses.command('edit-create', help='Create course using editor')
@click.option('-r', '--remote', required=False, help='Sets remote')
def cli_courses_create(obj, remote):
    remote: Remote = helpers.get_remote(obj, remote)
    params = dict(name='', codename='', description='')
    helpers.generic_edit_create(remote.kontr_client.courses.create, params)


@cli_courses.command('create', help='Create course')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-N', '--name', prompt=True, required=False, help='Name')
@click.option('-n', '--codename', prompt=True, required=False, help='Codename')
@click.option('-d', '--description', prompt=True, required=False, help='Description')
def cli_courses_create(obj, remote, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_create(remote.kontr_client.courses.create, **kwargs)


@cli_courses.command('select', help='Select course')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.argument('codename')
@click.pass_obj
def cli_courses_select(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"\nSelecting course: {codename} for remote: {remote.name}\n")
    remote.selected_course = codename
    remote.update()
    obj.remotes.save()


@cli_courses.command('deselect', help='Select course')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.pass_obj
def cli_courses_deselect(obj: AppConfig, remote):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"\nDeselecting course for the remote: {remote.name}\n")
    del remote.selected_course
    remote.update()
    obj.remotes.save()


@cli_courses.command('is-token-read', help='Get IS MU token for a course')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_courses_is_token_read(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course: resources.Course = remote.kontr_client.courses[codename]
    token = course.read_is_api_token()
    print(f"Token: {token}")


@cli_courses.command('is-token-update', help='Set IS MU token for a course')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.argument('token')
@click.pass_obj
def cli_courses_is_token_update(obj: AppConfig, remote, codename: str, token):
    remote: Remote = helpers.get_remote(obj, remote)
    course: resources.Course = remote.kontr_client.courses[codename]
    course.update_is_api_token(token=token)



import click

from kontrctl.config import AppConfig
from kontrctl.utils import helpers
import kontr_api


@click.command('info', help="Get info")
@click.pass_obj
def cli_info(obj: AppConfig):
    print("Kontrctl - Tool control the Kontr 2.0\n")
    print(f"Version: {obj.version}")
    print(f"Kontr-Api Version: {kontr_api.__version__}\n")
    remote_name = obj.config.default_remote
    if remote_name:
        _remote_info(obj, remote_name)


def _remote_info(obj, remote_name: str):
    print(f"Selected remote: {remote_name}")
    remote = helpers.get_remote(obj, remote_name)

    def __get_info(message, selector):
        if selector:
            print(f"{message}: {selector}")

    __get_info('Selected course', remote.selected_course)
    __get_info('Selected project', remote.selected_project)
    __get_info('Logged in user', remote.config.get('username'))
    __get_info('Logged in using identifier', remote.config.get('identifier'))



import click

from kontrctl.commands.project_configs import cli_project_configs
from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_projects = click.Group('projects', help='Projects management')

cli_projects.add_command(cli_project_configs)


@cli_projects.command('list', help='List all projects')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-c', '--course', required=False, help='Select course')
@click.pass_obj
def cli_projects_list(obj: AppConfig, remote, course):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    projects = course.projects.list()
    helpers.generic_list(projects, params=['id', 'name', 'codename', 'course.codename'])


@cli_projects.command('read', help='Get a project info')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_projects_read(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_read(course.projects, codename)


@cli_projects.command('rm', help='Delete project')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_projects_delete(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    print(f"Removing project: {codename}")
    course.projects.delete(codename)


@cli_projects.command('select', help='Select project')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_projects_select(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    print(f"\nSelecting project: {codename} for remote course {course.name} using {remote.name}\n")
    remote.selected_project = codename
    remote.update()


@cli_projects.command('create', help='Create project')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.option('-N', '--name', prompt=True, required=False, help='Name')
@click.option('-n', '--codename', prompt=True, required=False, help='Codename')
@click.option('-d', '--description', prompt=True, required=False, help='Description')
@click.option('-A', '--assignment-url', prompt=True, required=False, help='Assignment url')
def cli_projects_create(obj, remote, course, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_create(course.projects.create, **kwargs)


@cli_projects.command('deselect', help='Deselect project')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.pass_obj
def cli_projects_deselect(obj: AppConfig, remote):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"\nDeselecting project for remote {remote.name}\n")
    del remote.selected_project
    remote.update()


@cli_projects.command('edit', help='Edit project')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_projects_edit(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_edit(course.projects, codename)


@cli_projects.command('refresh-files', help='Refresh project files')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_projects_refresh_files(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    print(f"Refreshing project config: {course['codename']}/{codename}")
    course.projects.refresh_tests(codename)



import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_roles = click.Group('roles', help='Roles management')


@cli_roles.command('list', help='List all roles')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.pass_obj
def cli_roles_list(obj: AppConfig, remote, course):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    roles = course.roles.list()
    helpers.generic_list(roles, params=['id', 'name', 'course.codename'])


@cli_roles.command('read', help='Get a role info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_roles_read(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_read(course.roles, codename)


@cli_roles.command('rm', help='Delete role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_roles_delete(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    print(f"Deleting the role: {codename}")
    course.roles.delete(codename)


@cli_roles.command('edit', help='Edit roles')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_roles_edit(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_edit(course.roles, codename)


@cli_roles.command('create', help='Create role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.option('-N', '--name', prompt=True, required=False, help='Name')
@click.option('-n', '--codename', prompt=True, required=False, help='Codename')
@click.option('-d', '--description', prompt=True, required=False, help='Description')
def cli_roles_create(obj, remote, course, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_create(course.roles.create, **kwargs)


#
# Role users
#

cli_role_users = click.Group('clients', help='Clients management for the role')
cli_roles.add_command(cli_role_users)


@cli_role_users.command('list', help='List clients for the role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_role_users_list(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_list_users(course.roles, codename, name='clients')


@cli_role_users.command('add', help='Add client to the role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('client')
@click.pass_obj
def cli_role_users_add(obj: AppConfig, remote, course, codename: str, client: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_add_client(course.roles, codename, client)


@cli_role_users.command('rm', help='Remove client from the role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('client')
@click.pass_obj
def cli_role_users_add(obj: AppConfig, remote, course, codename: str, client: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_remove_client(course.roles, codename, client)







import click

from kontrctl import errors
from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers
from kontrctl.utils.helpers import entity_printer


@click.group('remotes', help='Manages remotes')
def cli_remotes():
    pass


@cli_remotes.command('add', help='Adds remote')
@click.argument('name', default='default')
@click.argument('url')
@click.pass_obj
def cli_remotes_add(obj: AppConfig, name, url):
    config = {}
    obj.remotes.add(name, url, **config)
    obj.remotes.save()


@cli_remotes.command('rm', help='Removes remote')
@click.argument('name')
@click.pass_obj
def cli_remotes_rm(obj: AppConfig, name: str):
    obj.remotes.delete(name)
    obj.remotes.save()


@cli_remotes.command('read', help='Reads remote')
@click.option('--output-format', default="text", help="Output format: (json|text|dict)")
@click.argument('name')
@click.pass_obj
def cli_remotes_read(obj: AppConfig, name: str, output_format):
    remote = obj.remotes[name]
    if remote is None:
        raise errors.ResourceNotFoundError(collection_name=remote.__class__.__name__, name=name)
    entity_printer(remote, output_format=output_format)


@cli_remotes.command('list', help='List remotes')
@click.pass_obj
def cli_remotes_list(obj: AppConfig):
    helpers.generic_list(obj.remotes.list(), params=['name', 'url'])


@cli_remotes.command('select', help='Selects remote')
@click.argument('name')
@click.pass_obj
def cli_select_remote(obj: AppConfig, name: str):
    obj.config.default_remote = name
    obj.save()


@cli_remotes.command('edit', help='Edit remotes info')
@click.argument('name')
@click.pass_obj
def cli_remote_edit(obj: AppConfig, name: str):
    remote: Remote = helpers.get_remote(obj, name)
    update_dict = helpers.edit_entity_using_editor(remote)
    if update_dict is not None:
        remote.config.update(update_dict)
        remote.update()
    print(update_dict)



import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_project_configs = click.Group('config', help='Projects configs management')


@cli_project_configs.command('read', help='Get a project config info')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_project_configs_read(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    project = helpers.read_helper(course.projects, codename)
    helpers.generic_read(project.project_config, codename)


@cli_project_configs.command('edit', help='Edit project config info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_project_configs_edit(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    project = course.projects.read(codename)
    project_config = project.project_config.read()
    update_dict = helpers.edit_entity_using_editor(project_config)
    if update_dict is not None:
        project.project_config.update(update_dict)
    print(update_dict)



import logging

import click
import kontr_api

import kontrctl
from kontrctl import log_config
from kontrctl.commands import add_all_commands
from kontrctl.config import AppConfig

log = logging.getLogger('kontrctl.cli')


@click.group(help='Kontr portal Cli tool')
@click.version_option(kontrctl.__version__)
@click.pass_context
def cli_main(ctx=None):
    import os
    if os.getenv('KONTR_LOG'):
        log_config.load_logger()
    log.info(f"KONTRCTL Version: {kontrctl.__version__} "
             f"KONTR-API Version: {kontr_api.__version__}")
    ctx.obj = AppConfig()


add_all_commands(cli_main)

if __name__ == '__main__':
    cli_main()



import click
from kontr_api.resources import Role

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_permissions = click.Group('permissions', help='Permissions management')


@cli_permissions.command('read', help='Get a permission info for the role by it\'s codename')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_permissions_read(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    role: Role = helpers.read_helper(course.roles, codename)
    helpers.entity_printer(role.permissions.read())


@cli_permissions.command('edit', help='Edit permissions for the role')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_permissions_edit(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    role: Role = helpers.read_helper(course.roles, codename)
    permissions = role.permissions.read()
    helpers.edit_and_update_entity(permissions)



import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers
from kontrctl.utils.helpers import entity_printer

cli_groups = click.Group('groups', help='Groups management')


@cli_groups.command('list', help='List all groups')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.pass_obj
def cli_groups_list(obj: AppConfig, remote, course):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    groups = course.groups.list()
    helpers.generic_list(groups, params=['id', 'name', 'course.codename'])



@cli_groups.command('read', help='Get a group info')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_groups_read(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_read(course.groups, codename)


@cli_groups.command('rm', help='Delete group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_groups_delete(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    print(f"Removing group: {codename}")
    course.groups.delete(codename)


@cli_groups.command('edit', help='Edit group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_groups_edit(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_edit(course.groups, codename)


@cli_groups.command('create', help='Create group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.option('-N', '--name', prompt=True, required=False, help='Name')
@click.option('-n', '--codename', prompt=True, required=False, help='Codename')
@click.option('-d', '--description', prompt=True, required=False, help='Description')
def cli_groups_create(obj, remote, course, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_create(course.groups.create, **kwargs)


#
# Group users
#

cli_group_users = click.Group('users', help='Users management for the group')
cli_groups.add_command(cli_group_users)


@cli_group_users.command('list', help='List users for the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_group_users_list(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_list_users(course.groups, codename)


@cli_group_users.command('add', help='Add user for the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('user')
@click.pass_obj
def cli_group_users_add(obj: AppConfig, remote, course, codename: str, user: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_add_client(course.groups, codename, user)


@cli_group_users.command('rm', help='Removes user from the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('user')
@click.pass_obj
def cli_group_users_add(obj: AppConfig, remote, course, codename: str, user: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_remove_client(course.groups, codename, user)


#
# Group projects
#

cli_group_projects = click.Group('projects', help='Project management for the group')
cli_groups.add_command(cli_group_projects)


@cli_group_projects.command('list', help='List projects for the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.pass_obj
def cli_group_projects_list(obj: AppConfig, remote, course, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_list_projects(course.groups, codename)


@cli_group_projects.command('add', help='Add project for the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('project')
@click.pass_obj
def cli_group_projects_add(obj: AppConfig, remote, course, codename: str, project: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_add_project(course.groups, codename, project)


@cli_group_projects.command('rm', help='Removes project from the group')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Select course')
@click.argument('codename')
@click.argument('project')
@click.pass_obj
def cli_group_projects_add(obj: AppConfig, remote, course, codename: str, project: str):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    helpers.generic_remove_project(course.groups, codename, project)



import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers


@click.command('submit', help='Create new submission')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-c', '--course', required=False, help='Sets course')
@click.option('-p', '--project', required=False, help='Sets project')
@click.option('-t', '--source-type', required=False, default='git',
              help='Sets type of submission files source (git)')
@click.option('-u', '--url', help='Source url - i.e. git repo url')
@click.option('-b', '--branch', required=False,
              help='Source branch - default: master')
@click.option('--checkout', help='Checkout param', required=False)
@click.option('-D', '--directory', help='Submission subdir', required=False)
@click.pass_obj
def cli_submit(obj: AppConfig, remote, course, project, source_type, url, branch, checkout, directory):
    remote: Remote = helpers.get_remote(obj, remote)
    course = helpers.get_course(remote, course)
    project = helpers.get_project(remote, course, project)
    file_params = _create_file_params(source_type, url, branch, checkout, directory)
    config = dict(file_params=file_params, project_params={})
    submission = project.submit(config)
    helpers.entity_printer(submission)


def _create_file_params(source_type, url: str, branch: str,
                        checkout: str, directory: str):
    source = dict(type=source_type, url=url)
    if branch:
        source['branch'] = branch
    if checkout:
        source['checkout'] = checkout
    file_params = dict(source=source, from_dir=directory)
    return file_params



import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_users = click.Group('users', help='Users management')


@cli_users.command('list', help='List all users')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('--ids', required=False, is_flag=True, help='Print just ids')
@click.option('--usernames', required=False, is_flag=True, help='Print just usernames')
@click.pass_obj
def cli_users_list(obj: AppConfig, remote, ids, usernames):
    remote: Remote = helpers.get_remote(obj, remote)
    users = remote.kontr_client.users.list()
    if ids or usernames:
        for user in users:
            if ids and usernames:
                print(f"{user['username']} {user['id']}")
            elif ids:
                print(user['id'])
            elif usernames:
                print(user['username'])
    else:
        params = ['id', 'username', 'email', 'uco']
        helpers.generic_list(users, params=params)


@cli_users.command('read', help='Get a user info')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.argument('username', required=False)
@click.pass_obj
def cli_users_read(obj: AppConfig, remote, username: str):
    remote: Remote = helpers.get_remote(obj, remote)
    user = helpers.get_user(remote, username=username)
    helpers.entity_printer(user)


@cli_users.command('rm', help='Delete user')
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.argument('username')
@click.pass_obj
def cli_users_delete(obj: AppConfig, remote, username: str):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Removing user: {username}")
    remote.kontr_client.users.delete(username)


@cli_users.command('create', help='Create user')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-u', '--username', prompt=True, required=False, help='Username')
@click.option('-n', '--name', prompt=True, required=False, help='Name')
@click.option('-e', '--email', prompt=True, required=False, help='Email')
@click.option('-U', '--uco', prompt=True, required=False, help='Uco')
@click.option('-A', '--admin', is_flag=True, required=False, help='Is Admin?')
@click.pass_obj
def cli_users_create(obj, remote, **kwargs):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_create(remote.kontr_client.users.create, **kwargs)


@cli_users.command('edit-create', help='Create users using editor')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.pass_obj
def cli_users_create_edit(obj, remote):
    remote: Remote = helpers.get_remote(obj, remote)
    params = {
        'username': '',
        'name': '',
        'email': '',
        'uco': '',
        'admin': False
    }
    helpers.generic_edit_create(remote.kontr_client.users.create, params)


@cli_users.command('edit', help='Edit user')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('username')
@click.pass_obj
def cli_users_edit(obj: AppConfig, remote, username: str):
    remote: Remote = helpers.get_remote(obj, remote)
    helpers.generic_edit(remote.kontr_client.users, username)


@cli_users.command('set-password', help='Set password for the user')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('username', required=False)
@click.pass_obj
def cli_users_update_password(obj: AppConfig, remote, username: str):
    remote: Remote = helpers.get_remote(obj, remote)
    user = helpers.get_user(remote, username)
    print(f"Updating password for {user['username']}")
    current = click.prompt('Current password', hide_input=True)
    new_password = click.prompt('New password', hide_input=True, confirmation_prompt=True)
    user.set_password(new=new_password, current=current)



from typing import Dict

import click
import yaml

from kontrctl import errors
from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_definitions = click.Group('definition', help='Work with resource definitions')


@cli_definitions.command('dump-course', help='Get a course definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_course_dump(obj: AppConfig, remote, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    dump = remote.kontr_client.definitions.dump_course(codename).decode('utf-8')
    print(dump)


@cli_definitions.command('dump-project', help='Get a project definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_project_dump(obj: AppConfig, remote, course: str, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    dump = remote.kontr_client.definitions.dump_project(course, codename).decode('utf-8')
    print(dump)


@cli_definitions.command('dump-role', help='Get a role definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_project_dump(obj: AppConfig, remote, course: str, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    dump = remote.kontr_client.definitions.dump_role(course, codename).decode('utf-8')
    print(dump)


@cli_definitions.command('dump-group', help='Get a group definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_project_dump(obj: AppConfig, remote, course: str, codename: str):
    remote: Remote = helpers.get_remote(obj, remote)
    dump = remote.kontr_client.definitions.dump_role(course, codename).decode('utf-8')
    print(dump)


@cli_definitions.command('sync-course', help='Sync a course using the definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-f', '--definition', required=False, help='Definition file - if not provided, '
                                                         'stdout is used')
@click.pass_obj
def cli_course_sync(obj: AppConfig, remote, definition=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = helpers.input_read(definition)
    if not schema:
        print(f"No schema content provided")
    course = remote.kontr_client.definitions.sync_course(schema).decode('utf-8')
    helpers.entity_printer(course)


@cli_definitions.command('sync-project', help='Sync a project using the definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-f', '--definition', required=False, help='Definition file - if not provided, '
                                                         'stdout is used')
@click.argument('course')
@click.pass_obj
def cli_project_sync(obj: AppConfig, remote, definition=None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = helpers.input_read(definition)
    if not schema:
        print(f"No schema content provided")
        return
    course = remote.kontr_client.definitions.sync_project(course, schema)
    helpers.entity_printer(course)


@cli_definitions.command('sync-role', help='Sync a role using the definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-f', '--definition', required=False, help='Definition file - if not provided, '
                                                         'stdout is used')
@click.argument('course')
@click.pass_obj
def cli_role_sync(obj: AppConfig, remote, definition=None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = helpers.input_read(definition)
    if not schema:
        print(f"No schema content provided")
        return
    course = remote.kontr_client.definitions.sync_role(course, schema)
    helpers.entity_printer(course)


@cli_definitions.command('sync-group', help='Sync a group using the definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-f', '--definition', required=False, help='Definition file - if not provided, '
                                                         'stdout is used')
@click.argument('course')
@click.pass_obj
def cli_group_sync(obj: AppConfig, remote, definition=None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = helpers.input_read(definition)
    if not schema:
        print(f"No schema content provided")
        return
    course = remote.kontr_client.definitions.sync_group(course, schema)
    helpers.entity_printer(course)


@cli_definitions.command('edit-course', help='Edit a course\'s definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('codename')
@click.pass_obj
def cli_course_edit(obj: AppConfig, remote, codename: str = None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = remote.kontr_client.definitions.dump_course(codename).decode('utf-8')
    serialized = _edit_schema(schema)
    course = remote.kontr_client.definitions.sync_course(serialized)
    helpers.entity_printer(course)


@cli_definitions.command('edit-project', help='Edit a project\'s definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_course_edit(obj: AppConfig, remote, codename: str = None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = remote.kontr_client.definitions.dump_project(course, codename).decode('utf-8')
    serialized = _edit_schema(schema)
    course = remote.kontr_client.definitions.sync_project(course, serialized)
    helpers.entity_printer(course)


@cli_definitions.command('edit-role', help='Edit a role\'s definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_course_edit(obj: AppConfig, remote, codename: str = None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = remote.kontr_client.definitions.dump_role(course, codename).decode('utf-8')
    serialized = _edit_schema(schema)
    course = remote.kontr_client.definitions.sync_role(course, serialized)
    helpers.entity_printer(course)


@cli_definitions.command('edit-group', help='Edit a group\'s definition')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('course')
@click.argument('codename')
@click.pass_obj
def cli_course_edit(obj: AppConfig, remote, codename: str = None, course=None):
    remote: Remote = helpers.get_remote(obj, remote)
    schema = remote.kontr_client.definitions.dump_group(course, codename).decode('utf-8')
    serialized = _edit_schema(schema)
    course = remote.kontr_client.definitions.sync_group(course, serialized)
    helpers.entity_printer(course)


def _edit_schema(schema: str) -> Dict:
    if not schema:
        print(f"No schema content provided")
    edited = click.edit(schema)
    if edited is None:
        raise errors.ContentNotChangedError()
    serialized = yaml.safe_load(edited)
    return serialized



import click


class KontrCtlError(click.ClickException):
    pass


class ResourceNotFoundError(KontrCtlError):
    def __init__(self, collection_name: str, name: str):
        super().__init__(f"Cannot find resource in {collection_name} with identifier: {name}")


class RemoteNotSetError(KontrCtlError):
    def __init__(self):
        super().__init__(f"Remote is not set.")


class ContentNotChangedError(KontrCtlError):
    def __init__(self):
        super(ContentNotChangedError, self).__init__("Content was not edited - no update")


class LoginNotSuccessfulError(KontrCtlError):
    def __init__(self):
        super(LoginNotSuccessfulError, self).__init__("Login was not successful")



import json
import sys

import click
import yaml
from kontr_api.resources import Course, Project
from tabulate import tabulate

from kontrctl import errors
from kontrctl.config import AppConfig, Remote


def get_remote(app_config: AppConfig, remote: str) -> Remote:
    """Gets remote instance
    Args:
        app_config(AppConfig): App config instance
        remote(str): Remote name
    Returns(Remote): Remote instance
    """
    remote_name = remote or app_config.config.default_remote
    remote = app_config.remotes[remote_name]
    if not remote:
        raise errors.RemoteNotSetError()
    return remote


def get_course(remote: Remote, course: str) -> Course:
    """Gets course based either by name or remote selected
    Args:
        remote(Remote): Remote instance
        course(str): Course name
    Returns(Course): Course instance
    """
    course_name = course or remote.selected_course
    return read_helper(remote.kontr_client.courses, course_name)


def get_project(remote: Remote, course: Course, project: str):
    """Gets project instance
    Args:
        remote(Remote): Remote instance
        course(Course): Course instance
        project(str): Project name

    Returns(Project): Project instance

    """
    project_name = project or remote.selected_project
    return read_helper(course.projects, project_name)


def get_user(remote: Remote, username: str = None):
    username = username or remote['username']
    if username is None:
        return remote.kontr_client.clients.me()
    return read_helper(remote.kontr_client.users, username)


def get_my_latest_submissions(remote: Remote) -> list:
    course = remote.selected_course
    project = remote.selected_project
    user = get_user(remote=remote)
    params = dict(user=user['username'])
    if course is not None:
        params['course'] = course
        if project is not None:
            params['project'] = project
    return remote.kontr_client.submissions.list(params=params)


def get_submission(remote: Remote, sid: str = None):
    if sid is not None:
        return read_helper(remote.kontr_client.submissions, sid)
    submissions = get_my_latest_submissions(remote)
    if not submissions:
        return None
    return submissions[-1]


def entity_printer(entity, output_format: str = 'text'):
    """Prints entity info
    Args:
        entity: Entity instance
        output_format(str): Format in which the entity will be printed (dict|json|text)
            (default: text)
    """
    if entity is None:
        return
    print(f"\n{entity.__class__.__name__} info:\n")
    config = entity.config
    if output_format == 'text' or output_format == 'yaml':
        print(yaml.safe_dump(config, sort=False))
    elif output_format == 'json':
        print(json.dumps(config))
    else:
        print(config)


def edit_entity_using_editor(entity) -> dict:
    """Edits config using the editor
    Args:
        entity: Entity instance, should have config property
    Returns(dict): Updated dict
    """
    json_content = json.dumps(entity.config, indent=4)
    edited = click.edit(json_content)
    if edited is None:
        raise errors.ContentNotChangedError()
    update_dict = json.loads(edited)
    return update_dict


def read_helper(client, entity_name):
    entity = client.read(entity_name)
    if not entity:
        raise errors.ResourceNotFoundError(collection_name=client.__class__.__name__,
                                           name=entity_name)
    return entity


def generic_edit(client, entity_name):
    print(f"Editing {client.__class__.__name__}: {entity_name}")
    entity = read_helper(client=client, entity_name=entity_name)
    edit_and_update_entity(entity)


def edit_and_update_entity(entity):
    updated = edit_entity_using_editor(entity=entity)
    entity.update(updated)
    entity_printer(entity=entity)


def generic_read(client, name: str, output_format: str = 'text'):
    """Generic read function
    Args:
        client: Entity collection client instance
        name(str): Name of the entity
        output_format(str): Output format
    """
    entity = read_helper(client=client, entity_name=name)
    entity_printer(entity, output_format=output_format)


def generic_create(create, **params):
    print(f"Create user: {params}")
    result = create(params)
    print(f"Result: {result}")


def generic_edit_create(create, params):
    return None


def generic_add_client(client, codename, user, client_type=None):
    print(f"Adding client '{user}' from the {client.__class__.__name__} for {codename}")
    entity = read_helper(client, codename)
    entity.add_client(user, client_type=client_type)
    entity.read()
    entity_printer(entity)


def generic_add_users(client, codename, *users, client_type=None):
    print(f"Adding clients '{users}' from the {client.__class__.__name__} for {codename}")
    entity = read_helper(client, codename)
    entity.add_clients(*users, client_type=client_type)
    entity.read()
    entity_printer(entity)


def generic_remove_client(client, codename, user, client_type=None):
    print(f"Removing client '{user}' from the {client.__class__.__name__}: {codename}")
    entity = read_helper(client, codename)
    entity.remove_client(user, client_type=client_type)
    entity.read()
    entity_printer(entity)


def generic_remove_users(client, codename, users):
    print(f"Removing clients '{users}' from the {client.__class__.__name__}: {codename}")
    entity = read_helper(client, codename)
    entity.remove_clients(users)
    entity.read()
    entity_printer(entity)


def generic_add_project(client, codename, project):
    print(f"Adding project '{project}' from the {client.__class__.__name__} for {codename}")
    entity = read_helper(client, codename)
    entity.add_project(project)
    entity.read()
    entity_printer(entity)


def generic_add_projects(client, codename, *projects):
    print(f"Adding project '{projects}' from the {client.__class__.__name__} for {codename}")
    entity = read_helper(client, codename)
    entity.add_projects(*projects)
    entity.read()
    entity_printer(entity)


def generic_remove_project(client, codename, project):
    print(f"Remove project '{project}' from the {client.__class__.__name__}: {codename}")
    entity = read_helper(client, codename)
    entity.remove_project(project)
    entity.read()
    entity_printer(entity)


def generic_remove_projects(client, codename, *projects):
    print(f"Remove projects '{projects}' from the {client.__class__.__name__}: {codename}")
    entity = read_helper(client, codename)
    entity.remove_projects(*projects)
    entity.read()
    entity_printer(entity)


def generic_list_users(client, codename, name='users'):
    print(f"Listing users for {client.__class__.__name__}:  \"{codename}\"")
    entity = read_helper(client, codename)
    generic_list(entity[name], params=['id', 'codename', 'name'])


def generic_list_projects(client, codename):
    entity = read_helper(client, codename)

    def __trans(project: Project):
        return [project['id'], project['codename'], project['course']['codename']]

    headers = ['ID', 'CODENAME', 'COURSE']
    generic_list(entity['projects'], __trans, headers=headers)


def _print_ids(data):
    for item in data:
        print(item['id'])


def generic_list(data_list, params=None, options=None, t_func=None, headers=None):
    def __default_trans(item):
        res = []
        for param in params:
            key = param.split('.')
            elem = item
            if not isinstance(item, dict):
                elem = item.config
            res.append(nested_get(elem, key))
        return res

    if options and options['cols']:
        params = options['cols'].split(';')

    if not t_func:
        t_func = __default_trans
    if not headers:
        headers = [param.upper().replace('.', ' ') for param in params]

    data = data_transform(data_list, t_func)
    if options and (options['ids']):
        _print_ids(data_list)
        return
    format = 'simple' if not options or not options['t_format'] else options['t_format']

    print(tabulate(data, headers=headers, tablefmt=format))


def get_any_client(remote: Remote, user, worker):
    if user:
        return read_helper(remote.kontr_client.users, user)
    elif worker:
        return read_helper(remote.kontr_client.workers, worker)
    else:
        return remote.kontr_client.clients.me()


def data_transform(data_list, t_func):
    return [t_func(data) for data in data_list]


def nested_get(input_dict, nested_key):
    internal_dict_value = input_dict
    for k in nested_key:
        internal_dict_value = internal_dict_value.get(k, None)
        if internal_dict_value is None:
            return None
    return internal_dict_value


def filter_submissions(submissions, points=None, result=None, state=None, **kwargs):
    res = []
    for subm in submissions:
        if result and subm['result'] != result:
            continue
        if points and not eval(points, None, dict(p=subm['points'])):
            continue
        if state and subm['state'] != state:
            continue
        res.append(subm)
    return res


def input_read(def_file=None):
    if def_file is None:
        return sys.stdout.read()
    with open(def_file, 'r') as fd:
        return fd.read()



import json
import logging
from pathlib import Path
from typing import List

import click
from kontr_api import KontrClient

import kontrctl

log = logging.getLogger(__name__)


class Entity:
    def __init__(self):
        """Creates abstract entity
        """
        self._config = {}

    @property
    def config(self) -> dict:
        """Gets config instance
        Returns(dict): Dict instance
        """
        return self._config

    def __setitem__(self, key, value):
        self.config[key] = value

    def __getitem__(self, item):
        return self.config.get(item)


class Remote(Entity):
    def __init__(self, remotes: 'Remotes', name: str, url: str, **params):
        """Creates instance of the remote
        Args:
            remotes(Remotes): Remotes instance
            name(str): Name of the remote
            url(str): Url
            **params:
        """
        super().__init__()
        self.remotes = remotes
        self._client = None
        self.config['name'] = name
        self.config['url'] = url
        self.config.update(**params)

    def update(self, save: bool = True):
        """Updates remote
        """
        self.remotes[self.name] = self
        if save:
            self.remotes.save()

    @property
    def name(self) -> str:
        """Gets name
        Returns(str): Name
        """
        return self.config['name']

    @property
    def url(self) -> str:
        """Gets url
        Returns(str): Url
        """
        return self.config['url']

    @property
    def selected_course(self) -> str:
        """Gets selected course
        Returns(str): Selected course
        """
        return self.config.get('selected_course')

    @selected_course.setter
    def selected_course(self, value: str):
        """Sets selected course
        Args:
            value(str): Value instance
        Returns:
        """
        self.config['selected_course'] = value

    @property
    def selected_project(self) -> str:
        """Gets selected course
        Returns(str): Selected course
        """
        return self.config.get('selected_course')

    @selected_project.deleter
    def selected_project(self):
        del self.config['selected_project']

    @selected_course.deleter
    def selected_course(self):
        del self.config['selected_course']

    @selected_project.setter
    def selected_project(self, value: str):
        """Sets selected project
        Args:
            value(str): Value instance
        Returns:
        """
        self.config['selected_project'] = value

    @property
    def kontr_client(self) -> KontrClient:
        """Creates instance of the kontr client
        Returns(KontrClient):
        """
        if self._client is None:
            self._client = self._create_client()
        return self._client

    def __build_config(self) -> dict:
        """Builds config
        Returns(dict): configuration dictionary
        """
        cfg = {'url': self.url}
        if 'access_token' in self.config:
            cfg['access_token'] = self.config['access_token']
        for name in ['access_token', 'refresh_token', 'username', 'password', 'secret']:
            cfg = self.__add_if_not_null(cfg, name)
        return cfg

    def __add_if_not_null(self, cfg: dict, name: str) -> dict:
        """Adds if not null
        Args:
            cfg(dict): Dict instance
            name(str): Name of the entity
        Returns(dict): Updated dict
        """
        if name in self.config:
            cfg[name] = self.config[name]
        return cfg

    def _create_client(self) -> KontrClient:
        """Creates new client instance
        Returns(KontrClient): Kontr client instance
        """
        config = self.__build_config()
        log.debug("Config has been built: %s", config)
        return KontrClient(**config)


class AbstractConfig:
    def __init__(self, fspath: Path = None):
        self._config = None
        self._fspath = fspath

    @property
    def config(self):
        if self._config is None:
            self._config = self._load()
        return self._config

    def load(self):
        self._config = self._load()

    def _load(self):
        if not self._fspath.exists():
            return {}
        with self._fspath.open() as config_file:
            loaded = json.load(config_file)
            log.debug('Loading file (%s): %s', self._fspath, loaded)
            return self.serialize(loaded)

    def save(self):
        if self._config is None:
            return
        if not self._fspath.parent.exists():
            self._fspath.parent.mkdir(parents=True)
        with open(str(self._fspath), 'w') as config_file:
            config_to_save = self.to_dict()
            log.debug("Saving file (%s): %s", self._fspath, config_to_save)
            json.dump(config_to_save, config_file, indent=4)

    def to_dict(self):
        return self.config

    def serialize(self, json_dict: dict) -> dict:
        return json_dict

    def __getitem__(self, item):
        return self.config.get(item)

    def __setitem__(self, key, value):
        self.config[key] = value


class UserConfig(AbstractConfig):
    @property
    def default_remote(self):
        return self.config.get('default_remote')

    @default_remote.setter
    def default_remote(self, remote_name):
        self.config['default_remote'] = remote_name


class Remotes(AbstractConfig):
    def add(self, name, url, **params):
        """Adds remote
        Args:
            name(str): Name of the remote
            url(str): Url to remote
            **params: Optional args

        """
        log.debug("[REMOTE] Adding remote: %s (%s): {%s}", name, url, params)
        self.config[name] = Remote(self, name, url, **params)

    def list(self) -> List[Remote]:
        """List of remotes
        Returns(list): List of remotes
        """
        log.debug("[REMOTE] Listing remotes")
        items = self.config.items()
        return [val for (key, val) in items]

    def delete(self, name):
        """Deletes remote
        Args:
            name(str): Name of the remote
        """
        log.debug("[REMOTE] Deleting remote: %s", name)
        del self.config[name]

    def serialize(self, json_dict: dict) -> dict:
        """Serializes json dict to object
        Args:
            json_dict(dict):
        Returns(dict):
        """
        res = {}
        for (key, val) in json_dict.items():
            config = dict(name=key)
            config.update(val)
            res[key] = Remote(self, **val)
        return res

    def to_dict(self) -> dict:
        """Serializes entity to dict
        Returns(dict):

        """
        result = {}
        for (name, remote) in self.config.items():
            result[name] = remote.config
        return result


class AppConfig:
    APP_NAME = 'kontrctl'

    def __init__(self):
        """Creates application config instance
        """
        user_config = Path(click.get_app_dir(AppConfig.APP_NAME))
        self._config = UserConfig(user_config / 'user_config.json')
        self._remotes = Remotes(user_config / 'remotes.json')

    @property
    def config(self) -> UserConfig:
        """Gets user config instance
        Returns(UserConfig): User config instance
        """
        return self._config

    @property
    def remotes(self) -> Remotes:
        """Gets remote config
        Returns(Remotes): Remotes instance
        """
        return self._remotes

    @property
    def version(self) -> str:
        """Gets current version of the Kontrctl
        Returns(str): Version string
        """
        return kontrctl.__version__

    def save(self):
        """Saves all the configs
        """
        self.config.save()
        self.remotes.save()



import re
from pathlib import Path

from setuptools import find_packages, setup

long_description = Path('README.md').read_text(encoding='utf-8')

VERSION = re.search(r'__version__ = \'(.*?)\'',
                    Path('kontrctl/__init__.py').read_text(encoding='utf-8')
                    ).group(1)

requirements = ['click', 'kontr-api', 'tabulate']

extras = {
    'dev': [
        'pytest',
        'coverage',
        'mock',
    ],
    'docs': ['sphinx']
}

classifiers = [
    "Programming Language :: Python :: 3",
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    "Operating System :: OS Independent",
    "License :: OSI Approved :: Apache Software License",
    'Intended Audience :: Developers',
    'Topic :: Utilities',
]

entry_points = {
    'console_scripts': [
        'kontrctl = kontrctl.cli:cli_main',
    ],
}

setup(name='kontrctl',
      version=VERSION,
      description='Kontr portal CLI',
      author='Peter Stanko',
      author_email='stanko@mail.muni.cz',
      url='https://gitlab.fi.muni.cz/grp-kontr2/kontrctl',
      packages=find_packages(exclude=("tests",)),
      long_description=long_description,
      long_description_content_type='text/markdown',
      include_package_data=True,
      install_requires=requirements,
      extras_require=extras,
      entry_points=entry_points,
      classifiers=classifiers,
      )



from .definitions import cli_definitions
from .auth import cli_login, cli_logout
from .courses import cli_courses
from .groups import cli_groups
from .general import cli_info
from .permissions import cli_permissions
from .project_configs import cli_project_configs
from .projects import cli_projects
from .remotes import cli_remotes
from .roles import cli_roles
from .secrets import cli_secrets
from .submit import cli_submit
from .submission_files import cli_submission_files
from .submissions import cli_submissions
from .users import cli_users
from .workers import cli_workers
from .execute import cli_shell


def add_all_commands(main_command):
    main_command.add_command(cli_remotes)
    main_command.add_command(cli_login)
    main_command.add_command(cli_logout)
    main_command.add_command(cli_users)
    main_command.add_command(cli_workers)
    main_command.add_command(cli_courses)
    main_command.add_command(cli_projects)
    main_command.add_command(cli_roles)
    main_command.add_command(cli_groups)
    main_command.add_command(cli_submit)
    main_command.add_command(cli_submissions)
    main_command.add_command(cli_permissions)
    main_command.add_command(cli_secrets)
    main_command.add_command(cli_info)
    main_command.add_command(cli_shell)
    main_command.add_command(cli_definitions)




import logging

import click

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

log = logging.getLogger(__name__)


@click.command('shell',
               help="Start the shell - IPython is currently supported - needs to be installed")
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.pass_obj
def cli_shell(obj: AppConfig, remote: str):
    remote: Remote = helpers.get_remote(obj, remote)
    try:
        _load_shell(remote)
    except ImportError:
        print("You have not installed the IPython package,\n"
              "to solve this problem please run:\n\n"
              "$ pip install ipython\n\nIn order to be able to use this feature")


def _load_shell(remote: Remote):
    import IPython
    kontr_client = remote.kontr_client
    IPython.embed()



import json
from pathlib import Path

import click
from kontr_api.resources import Submission

from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

cli_submission_files = click.Group('files', help='Submission files management')


def general_source_getter(submission, entity_name, path, out):
    client = getattr(submission, entity_name)
    if path is None:
        out = out or '.'
        out = Path(out) / f"{submission['id']}-{entity_name}.zip"
        client.download(out)
        print(f"Files downloaded: {out}")
    else:
        file_content = client.content(path)
        print(file_content)


@cli_submission_files.command('sources-tree', help='Get submission sources tree')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_sources(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    print(json.dumps(submission.sources.tree(), indent=4))


@cli_submission_files.command('results-tree', help='Get results tree')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_sources(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    print(json.dumps(submission.results.tree(), indent=4))


@cli_submission_files.command('test-files-tree', help='Get results tree')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_sources(obj: AppConfig, remote, sid: str):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    print(json.dumps(submission.test_files.tree(), indent=4))


@cli_submission_files.command('sources', help='Get submission sources')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-o', '--out', required=False, help='Set out path')
@click.option('-P', '--path', required=False, help='File path')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_sources(obj: AppConfig, remote, sid: str, out=None, path: str = None):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    general_source_getter(submission, 'sources', path=path, out=out)


@cli_submission_files.command('results', help='Get submission results')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-o', '--out', required=False, help='Set out path')
@click.option('-P', '--path', required=False, help='File path')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_results(obj: AppConfig, remote, sid: str, out=None, path: str = None):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    general_source_getter(submission, 'results', path=path, out=out)


@cli_submission_files.command('test-files', help='Get submission results')
@click.option('-r', '--remote', required=False, help='Sets remote')
@click.option('-o', '--out', required=False, help='Set out path')
@click.option('-P', '--path', required=False, help='File path')
@click.argument('sid', required=False)
@click.pass_obj
def cli_submission_files_test_files(obj: AppConfig, remote, sid: str, out=None, path: str = None):
    remote: Remote = helpers.get_remote(obj, remote)
    submission = helpers.get_submission(remote, sid)
    general_source_getter(submission, 'test_files', path=path, out=out)



"""
Logging configuration module
"""

from logging.config import dictConfig

FORMATTERS = {
    'verbose': {
        'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    },

    'simple': {
        'format': '%(levelname)s %(message)s'
    },
    'colored_console': {
        '()': 'coloredlogs.ColoredFormatter',
        'format': "%(asctime)s - %(name)s - %(levelname)s - %(message)s", 'datefmt': '%H:%M:%S'
    },

}

HANDLERS = {
    'console': {
        'level': 'DEBUG',
        'class': 'logging.StreamHandler',
        'formatter': 'colored_console'
    },
}

LOGGERS = {
    'tests': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'kontrctl': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'kontr_api': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
}

LOGGING_CONF = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': FORMATTERS,
    'handlers': HANDLERS,
    'loggers': LOGGERS,
}


def load_logger(conf_type=None):
    """Loads config based on the config type
    Args:
        conf_type(str): Config type available (dev, test, prod)

    """
    if conf_type is not None:
        print("Not implemented yet!")
    dictConfig(LOGGING_CONF)



import logging

import click
from kontr_api import KontrClient

from kontrctl import errors
from kontrctl.config import AppConfig, Remote
from kontrctl.utils import helpers

log = logging.getLogger(__name__)


@click.command('logout', help="Logout")
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-a', '--all', required=False, help='Remove all login info', 
              is_flag=True, default=False)
@click.pass_obj
def cli_logout(obj: AppConfig, remote: str, all: bool):
    remote: Remote = helpers.get_remote(obj, remote)
    print(f"Logout from: {remote.name} ({remote.url})")
    _logout(remote, all=all)
    helpers.entity_printer(remote)


def _logout(remote: Remote, all: bool):
    def __safe_del(remote: Remote, name: str):
        if name in remote.config:
            del remote.config[name]

    def __del_params(*params):
        for name in params:
            __safe_del(remote, name)

    __del_params('access_token', 'refresh_token')
    
    if all:
        __del_params('username', 'identifier', 'password', 'secret')
    remote.update()


@click.command('login', help="Login to portal")
@click.option('-r', '--remote', required=False, help='Sets remote for the login')
@click.option('-u', '--user', required=False, help='Sets user login')
@click.option('-s', '--secret', required=False, help='Sets user secret')
@click.option('--keep-credentials', required=False, default=False, is_flag=True,
              help='Sets user login to remotes file')
@click.pass_obj
def cli_login(obj: AppConfig, remote, user, secret, keep_credentials):
    remote = helpers.get_remote(obj, remote)
    print(f"Login to : {remote.name} ({remote.url})")
    credentials = ask_for_credentials(remote, user, secret)
    if keep_credentials:
        _save_credentials_to_remote(remote, **credentials)
    login_and_set_tokens(remote, **credentials)
    log.debug("Updated remote(%s): {%s}", remote.name, remote.config)
    helpers.entity_printer(remote)
    obj.save()


def ask_for_credentials(remote: Remote, user, secret) -> (str, str):
    user = user or remote['username']
    secret = secret or remote['secret']
    password = remote['password']
    if user is None:
        user = click.prompt('User')
    if password is None and secret is None:
        password = click.prompt('Password', hide_input=True)
    return dict(password=password, username=user, secret=secret, identifier=user)


def login_and_set_tokens(remote: Remote, **credentials) -> Remote:
    kontr: KontrClient = KontrClient(remote.url, **credentials)
    tokens = kontr.login()
    log.debug("[LOGIN] Tokens: %s", tokens)
    if 'access_token' not in tokens:
        raise errors.LoginNotSuccessfulError()
    remote['access_token'] = tokens['access_token']
    remote['refresh_token'] = tokens['refresh_token']
    remote.update()
    return remote


def _save_credentials_to_remote(remote: Remote, **credentials):
    for k, v in credentials.items():
        if v is not None:
            remote[k] = v
    remote.update()
    return remote
